/*****************************************************
CVAVR V2.05
Chip type               : ATmega8 DIP28
AVR Core Clock frequency: 8MHz
Project                 : Sinewave converter
Version                 : 0
*****************************************************/
#include <mega8.h>
#include "periph_init.c"
#include "defines.c"
#include "display_work.c"
#include <math.h>
#include "sw_calc.c"
#include "kwork.c"
#include "adc_work.c"

u16 mst=0;
bit mst_ready=1;
bit inv=1;
bit t1_iwdg;

volatile u16 tim0,
             tim1,
             tim2,
             tim3;
          


interrupt [ANA_COMP] void acmp_isr()
{
#ifdef ocp_control

 volatile u8 cnt;
 for(cnt=0;cnt<4;cnt++);
 if(ACSR.ACO){HKL=LKL=HKR=LKR=1;inv=0;} 
 
#endif
}
interrupt [TIM1_OVF] void t1_ovf()
{
 static u8 i=0;
 static bit n=0;
 static u16 i_mst=0;
 u16 tmp;     
 static u16 tim_cnt;
 
 i++;
 tmp = (u16)((u32)s_table[i]*i_mst>>8);
 if(tmp>pwm_top)tmp = pwm_top; 
 else if(tmp<DTV)tmp = DTV; 
 OCR1A=OCR1B = tmp;
  
 if(!inv)
 {
  i=0;
  HKL=LKL=HKR=LKR=1;
  TCCR1A = 0;
 }
 else if(i>=s_step-1)
 {
  HKR = 1;
  LKR = 1;
  if(n)
  {
   TCCR1A = 0b10110000;
   OCR1A+=DTV;
  }
  else 
  {
   TCCR1A = 0b11100000;
   OCR1B+=DTV;
  }
  i=0; 
  n^=1;
  if(mst_ready&&n)
  {
   i_mst=mst;
   mst_ready = 0;
  } 
 } 
 else
 {
  if(i>1)
  {
   if(n)LKR = 0;
   else HKR = 0;
  }
  else LKR=HKR = 1; 
    
  if(n)
  {
   TCCR1A = 0b10110000;
   if(OCR1A+DTV+DTV>pwm_top)OCR1A=OCR1B=pwm_top;
   else OCR1A+=DTV;  
  }
  else
  {
   TCCR1A = 0b11100000; 
   if(OCR1B+DTV+DTV>pwm_top)OCR1B=OCR1A=pwm_top;
   else OCR1B+=DTV;
  } 
 }

 if(tim_cnt<timcnttrs) tim_cnt++;
 else
 { 
  tim_cnt = 0;  
  if(tim0)tim0--;
  if(tim1)tim1--;
  if(tim2)tim2--;
  if(tim3)tim3--;
 }

 t1_iwdg=1;
}

void devcon();

void main(void)
{
 gpio_init();
 adc_init();
 acmp_init();

 detect_7seg_polarity(); 
 calculate_sw_tb();
 
 t1_init(quartz_frq/2/pwm_frq); 

 #asm("sei"); 
 
#ifndef debug
#pragma optsize-
WDTCR=0x18;
WDTCR=0x08;
#endif
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif 

 
m:
 { 
  display_data();
  scan_keys();    
  devcon();
  if(t1_iwdg){t1_iwdg=0; #asm("wdr")}
 } 
goto m;
}


void devcon()
{
 static bit itr=0,btn_trg; 
 static bit state;
 static enum {start,work,off,ocp,lobt,wait} dev_st=start;
 
 if(btn[0]==rls)btn_trg=0;

 switch (dev_st)
 {
  case start:
  {
   if(!itr)
   {
    itr=1;
    tim1=1000;
    inv=0;
   }
   
   static bit trg=0; 
   
   if(!tim1)
   {
    if(!trg)
    {
     trg=1;
     tim1=1000;
     dbuf[2]='r';dbuf[1]='u';dbuf[0]='n'; dpp=5;
    }
    else
    {
     u8 adc_tmp = scan_adc(&mst);
     
     if(adc_tmp)
     {
      static u8 lb_cnt=0;

      lb_cnt++;     
      if((u32)adc_tmp*adc_ub/0xff>LoBt_trs){dev_st=work;lb_cnt=0;}
      else if(!lb_cnt) dev_st=lobt;
     }     
    }
   }
   
   if(dev_st!=start){itr=0;trg=0;}
  }
  break;

  case work:
  { 
   u8 adc_tmp;
   if(!itr)
   {
    itr=1;
    inv=1;
    mst=0;
   }

#ifdef ocp_control 
   if(!inv)dev_st=ocp;
#endif

   adc_tmp = scan_adc(&mst);
   
   if(adc_tmp)
   {
    u16 vtmp = (u32)adc_tmp*adc_ub/0xff;
    if(!tim0)
    {
     tim0=1000;
     convert_data(vtmp/10); dpp=1;
    }    
    if(vtmp<800)dev_st=lobt;
    else if(vtmp>LoBt_trs)tim1=5*100;
    else if(vtmp<LoBt_trs && !tim1)dev_st=lobt;    
   }
   
   if(btn[0]==hold && !btn_trg){btn_trg=1;dev_st=off;}
   
#ifdef mains_control

   static u8 mcnt=0;
   
   if(!tim3 && mcnt>40){dev_st=wait;mcnt=0;}   
   if(!mains_ips && !tim3){tim3=1000;mcnt=0;}
   if(mains_ips!=state && tim3)
   {
    state=mains_ips;
    if(mcnt<0xff)mcnt++;
   } 
   
#endif   
   

   if(dev_st!=work){itr=0;}
  }
  break;

  case off:
  {
   if(!itr)
   {
    itr=1;
    inv=0;
    dbuf[2]='_';dbuf[1]='_';dbuf[0]='_'; dpp=5;
    tim1=5*100;    
   }
   
   if(btn[0]==hold && !btn_trg){btn_trg=1;dev_st=start;}
   
   //if(!tim1)H_keys=0;
 

   if(dev_st!=off){itr=0;}
  }
  break;

  case ocp:
  {
   if(!itr)
   {
    itr=1;
    dbuf[2]='O';dbuf[1]='C';dbuf[0]='P'; dpp=5;
    inv=0;
    tim1=5000;     
   }
   
   if(btn[0]==hold && !btn_trg){btn_trg=1;dev_st=start;}
   //if(!tim1)H_keys=0;   

   if(dev_st!=ocp){itr=0;}
  }
  break;  

  case lobt:
  {
   u8 adc_tmp;
   if(!itr)
   {
    itr=1;
    inv=0;
    dbuf[2]='L';dbuf[1]='o';dbuf[0]='b'; dpp=1;
    tim1=2000;
   }
   
   adc_tmp = scan_adc(&mst);
   
   if((btn[0]==hold||btn[0]==prs) && !btn_trg && adc_tmp)
   {    
    btn_trg=1;
    
    if((u32)adc_tmp*adc_ub/0xff>LoBt_trs)dev_st=start;
    else
    {
     tim1=2000;
     convert_data((u32)adc_tmp*adc_ub/0xff/10);
     dpp=1;
    } 
   }

   if(!tim1)
   {
    dbuf[2]='L';dbuf[1]='o';dbuf[0]='b'; dpp=1;
    //H_keys=0; 
   }  
   

   if(dev_st!=lobt){itr=0;}
  }
  break; 
  
  case wait:
  {
   if(!itr)
   {
    itr=1;
    inv=0;
    dbuf[2]=2;dbuf[1]=2;dbuf[0]=0; dpp=5;
    tim1=5000; 
    tim3=100;   
   }
   
  // if(!tim1)H_keys=0; 
   
   if(state!=mains_ips)
   {
    state = mains_ips;
    tim3=100;
   }  
       
   if(!tim3)dev_st=start;
 
   if(dev_st!=wait){itr=0;}
  }
  break;  

  default:break;
 }

}

